#include <GL/gl.h>

#include "glwidget.h"

//=================================================================================================
GLWidget::GLWidget( QWidget *parent ):
  QGLWidget( parent )
{
}
//=================================================================================================
GLWidget::~GLWidget()
{
  // удаляем текстурные объекты
  for (int i = 0; i < textures.count(); ++i)
    deleteTexture( textures.at(i).id );
}
//=================================================================================================
void GLWidget::initializeGL()
{
  // задаём линейную фильтрацию вблизи
  glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
  // задаём линейную фильтрацию вдали
  glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR );
  // далее параметры текстурного объекта
  // при фильтрации игнорируются тексели, выходящие за границу текстуры для s координаты
  glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE_MY );
  // при фильтрации игнорируются тексели, выходящие за границу текстуры для t координаты
  glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE_MY );
  // цвет текселя полностью замещает цвет фрагмента фигуры
  glTexEnvi( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE );

  // цвет фона
  glClearColor( 0.0f, 0.0f, 0.0f, 1.0f );
  // сгенерировать текстуру
  genTexture();
  // установить режим двумерных текстур
  glEnable( GL_TEXTURE_2D );
}
//=================================================================================================
void GLWidget::resizeGL( int w, int h )
{
  // устанавливаем текущей проекционную матрицу
  glMatrixMode( GL_PROJECTION );
  // присваиваем проекционной матрице единичную матрицу
  glLoadIdentity();
  // поле просмотра
  glViewport( 0, 0, w, h );
  // система координат
  glOrtho( -w / 2.0, w / 2.0, -h / 2.0, h / 2.0, -10.0, 10.0 );
}
//=================================================================================================
void GLWidget::paintGL()
{
  // очищаем буферы изображения и глубины
  glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
  // устанавливаем положение и ориентацию матрице моделирования
  glMatrixMode( GL_MODELVIEW );
  // загружаем единичную матрицу моделирования
  glLoadIdentity();
  // запоминаем матрицу
  glPushMatrix();
  // трансляция
  glTranslatef( 0.0f, 0.0f, 0.0f );

  // текущая текстура
  int curTexture = 0;

  // выводим все части изображения
  for (int y = 0; y < blocksY; ++y)
  {
    for (int x = 0; x < blocksX; ++x)
    {
      // ширина и высота части изображения
      int ww = textures.at( curTexture ).size.width();
      int hh = textures.at( curTexture ).size.height();

      // выбираем текстуру для наложения
      glBindTexture( GL_TEXTURE_2D, textures.at( curTexture ).id );

      // определяем координаты для вывода части изображения
      int xx = -sizeX / 2 + x * m_iMaxTexSize;
      int yy = sizeY / 2 - y * m_iMaxTexSize - hh;

      glBegin( GL_QUADS );
        glTexCoord2d( 0, 0 ); glVertex2d( xx, yy );           // нижний левый угол
        glTexCoord2d( 0, 1 ); glVertex2d( xx, yy + hh );      // верхний левый угол
        glTexCoord2d( 1, 1 ); glVertex2d( xx + ww, yy + hh ); // верхний правый угол
        glTexCoord2d( 1, 0 ); glVertex2d( xx + ww, yy );      // нижний правый угол
      glEnd();

      ++curTexture;
    }
  }

  // восстанавливаем матрицу
  glPopMatrix();
}
//=================================================================================================
void GLWidget::genTexture()
{
  img = new QImage("1280x1024.jpg");

  sizeX = img->width();
  sizeY = img->height();

  // размеры изображения
  sizeX = img->width();
  sizeY = img->height();

  // очищаем список текстур
  textures.clear();

  // максимальный размер текстуры
  glGetIntegerv( GL_MAX_RECTANGLE_TEXTURE_SIZE_ARB, &m_iMaxTexSize );

  // количество частей изображения по горизонтали
  if (sizeX >= m_iMaxTexSize)
    blocksX = sizeX / m_iMaxTexSize;

  if (sizeX % m_iMaxTexSize != 0)
    ++blocksX;

  // количество частей изображения по вертикали
  if (sizeY >= m_iMaxTexSize)
    blocksY = sizeY / m_iMaxTexSize;

  if (sizeY % m_iMaxTexSize != 0)
    ++blocksY;

  // ширина и высота частей изображения
  int ww = 0;
  int hh = 0;

  s_Textures texture;

  // определяем размеры частей изображения
  for (int y = 0; y < blocksY; ++y)
  {
    hh = sizeY - hh * y;
    ww = 0;

    for (int x = 0; x < blocksX; ++x)
    {
      ww = sizeX - ww * x;

      if (ww >= m_iMaxTexSize)
        ww = m_iMaxTexSize;

      if (hh >= m_iMaxTexSize)
        hh = m_iMaxTexSize;

      // копируем часть изображения
      QImage *tmpImg = new QImage( img->copy( m_iMaxTexSize * x, m_iMaxTexSize * y, ww, hh ) );
      // создаём, связываем, загружаем, возвращаем уникальный номер
      texture.id = bindTexture( tmpImg->convertToFormat( QImage::Format_ARGB32 ), GL_TEXTURE_2D, GL_RGBA );
      texture.size = tmpImg->size();
      // заносим текстуру в список
      textures.push_back( texture );

      delete tmpImg;
      tmpImg = 0;
    }
  }

  delete img;
  img = 0;
}
//=================================================================================================
